class InvalidSquare(Exception):
    pass

class InvalidPlayer(Exception):
    pass

class UnknownError(Exception):
    pass

class NotTheirTurn(Exception):
    pass

class CannotBeTheSame(Exception):
    pass

class SpaceOccupied(Exception):
    pass